@@ -6,7 +6,7 @@
  * This program is free software; you can redistribute it and/or modify it
  * under the terms and conditions of the GNU Lesser General Public License
  * as published by the Free Software Foundation; either version 2.1 of the
- * License, or (at your option0 any later version.
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope it will be useful, but WITHOUT ANY
  * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -55,6 +55,19 @@ static EnumDescription PPTextAlign_desc[] =
   { NULL,     0 }
 };
 
+static EnumDescription PPGravity_desc[] =
+{
+  { "center",       CLUTTER_GRAVITY_CENTER },
+  { "top-left",     CLUTTER_GRAVITY_NORTH_WEST },
+  { "left",         CLUTTER_GRAVITY_WEST },
+  { "bottom-left",  CLUTTER_GRAVITY_SOUTH_WEST },
+  { "center",       CLUTTER_GRAVITY_CENTER },
+  { "top-right",    CLUTTER_GRAVITY_NORTH_EAST },
+  { "right",        CLUTTER_GRAVITY_EAST },
+  { "bottom-right", CLUTTER_GRAVITY_SOUTH_EAST },
+  { NULL,     0 }
+};
+
 #define PINPOINT_RENDERER(renderer) ((PinPointRenderer *) renderer)
 
 /* pinpoint defaults */
@@ -64,6 +77,7 @@ static PinPointPoint pin_default_point = {
   .bg = NULL,
   .bg_type = PP_BG_NONE,
   .bg_scale = PP_BG_FIT,
+  .bg_position = CLUTTER_GRAVITY_CENTER,
 
   .text = NULL,
   .position = CLUTTER_GRAVITY_CENTER,
@@ -99,6 +113,7 @@ gboolean  pp_fullscreen      = FALSE;
 gboolean  pp_maximized       = FALSE;
 gboolean  pp_speakermode     = FALSE;
 gboolean  pp_rehearse        = FALSE;
+gboolean  pp_ignore_comments = FALSE;
 char     *pp_camera_device   = NULL;
 
 static GOptionEntry entries[] =
@@ -114,6 +129,8 @@ static GOptionEntry entries[] =
     "Show speakermode window", NULL},
     { "rehearse", 'r', 0, G_OPTION_ARG_NONE, &pp_rehearse,
     "Rehearse timings", NULL},
+    { "ignore-comments", 'i', 0, G_OPTION_ARG_NONE, &pp_ignore_comments,
+    "don't show comments", NULL},
     { "output", 'o', 0, G_OPTION_ARG_STRING, &pp_output_filename,
       "Output presentation to FILE\n"
 "                                         (formats supported: pdf)", "FILE" },
@@ -312,8 +329,42 @@ pp_get_background_position_scale (PinPointPoint *point,
       *bg_scale_y = h_scale;
       break;
     }
-  *bg_x = (stage_width - bg_width * *bg_scale_x) / 2;
-  *bg_y = (stage_height - bg_height * *bg_scale_y) / 2;
+
+  switch (point->bg_position)
+    {
+      case CLUTTER_GRAVITY_EAST:
+      case CLUTTER_GRAVITY_NORTH_EAST:
+      case CLUTTER_GRAVITY_SOUTH_EAST:
+        *bg_x = stage_width * 0.95 - bg_width * *bg_scale_x;
+        break;
+      case CLUTTER_GRAVITY_WEST:
+      case CLUTTER_GRAVITY_NORTH_WEST:
+      case CLUTTER_GRAVITY_SOUTH_WEST:
+        *bg_x = stage_width * 0.05;
+        break;
+      case CLUTTER_GRAVITY_CENTER:
+      default:
+        *bg_x = (stage_width - bg_width * *bg_scale_x) / 2;
+        break;
+    }
+
+  switch (point->bg_position)
+    {
+      case CLUTTER_GRAVITY_SOUTH:
+      case CLUTTER_GRAVITY_SOUTH_EAST:
+      case CLUTTER_GRAVITY_SOUTH_WEST:
+        *bg_y = stage_height * 0.95 - bg_height * *bg_scale_y;
+        break;
+      case CLUTTER_GRAVITY_NORTH:
+      case CLUTTER_GRAVITY_NORTH_EAST:
+      case CLUTTER_GRAVITY_NORTH_WEST:
+        *bg_y = stage_height * 0.05;
+        break;
+      case CLUTTER_GRAVITY_CENTER:
+      default:
+        *bg_y = (stage_height - bg_height * *bg_scale_y) / 2;
+        break;
+    }
 }
 
 void
@@ -468,6 +519,7 @@ parse_setting (PinPointPoint *point,
   IF_EQUAL("fit")          point->bg_scale = PP_BG_FIT;
   IF_EQUAL("stretch")      point->bg_scale = PP_BG_STRETCH;
   IF_EQUAL("unscaled")     point->bg_scale = PP_BG_UNSCALED;
+  IF_PREFIX("bg-position=") ENUM(point->bg_position, PPGravity, STRING);
   IF_EQUAL("center")       point->position = CLUTTER_GRAVITY_CENTER;
   IF_EQUAL("top")          point->position = CLUTTER_GRAVITY_NORTH;
   IF_EQUAL("bottom")       point->position = CLUTTER_GRAVITY_SOUTH;
@@ -601,6 +653,33 @@ static void serialize_slide_config (GString       *str,
         }
     }
 
+  if (point->bg_position != reference->bg_position)
+    {
+      g_string_append(str, separator);
+      switch (point->bg_position)
+        {
+          case CLUTTER_GRAVITY_NONE:
+          case CLUTTER_GRAVITY_CENTER:
+            break;
+          case CLUTTER_GRAVITY_NORTH:
+            g_string_append (str, "[bg-position=top]");break;
+          case CLUTTER_GRAVITY_SOUTH:
+            g_string_append (str, "[bg-position=bottom]");break;
+          case CLUTTER_GRAVITY_WEST:
+            g_string_append (str, "[bg-position=left]");break;
+          case CLUTTER_GRAVITY_EAST:
+            g_string_append (str, "[bg-position=right]");break;
+          case CLUTTER_GRAVITY_NORTH_WEST:
+            g_string_append (str, "[bg-position=top-left]");break;
+          case CLUTTER_GRAVITY_NORTH_EAST:
+            g_string_append (str, "[bg-position=top-right]");break;
+          case CLUTTER_GRAVITY_SOUTH_WEST:
+            g_string_append (str, "[bg-position=bottom-left]");break;
+          case CLUTTER_GRAVITY_SOUTH_EAST:
+            g_string_append (str, "[bg-position=bottom-right]");break;
+        }
+    }
+
   if (point->text_align != reference->text_align)
     {
       g_string_append (str, separator);
@@ -873,7 +952,7 @@ pp_parse_slides (PinPointRenderer *renderer,
               }
             break;
         case '#': /* comment */
-          if (startofline)
+          if (startofline && ! pp_ignore_comments)
             {
               const char *end = p + 1;
               while (*end != '\n' && *end != '\0')
